
import Dimensions;
import Polygon;
import Bounds;
import Theater;

import java.awt.Graphics;

public abstract class Actor
   {
   /*
   Define a class to represent actors in a theater.

   An Actor must be able to:
   -act() to change its appearance, position, or behavior,
   -isMobile() to report if its mobile or not,
   -getBounds() to return its rectangular boundaries,
   -handleCollision() with another actor or a boundary,
   -paint() itself on a given smartTablet.
   */

   //this actor acts in a theater
   private final Theater theater;

   public Actor(Theater theater)
      {
      /*
      Initialize this actor to act in the given theater.
      */

      this.theater = theater;
      }

   public abstract void act();
   public abstract boolean isMobile();
   public abstract Bounds getBounds();
   public abstract void handleCollision(Actor actor);
   public abstract void handleCollision(Polygon perimeter);
   public abstract void paint(Graphics smartTablet);

   protected final void removeActor(Actor actor)
      {
      /*
      Delete the named actor from the stage.
      */

      theater.removeActor(actor);
      }

   protected final void playSound(String soundName)
      {
      /*
      Play the named sound.
      */

      theater.playSound(soundName);
      }

   protected final Dimensions getStageDimensions()
      {
      /*
      Report this actor's stage's dimensions.
      */

      return theater.getStageDimensions();
      }

   protected final Polygon getPolygon()
      {
      /*
      Report a polygon representing the actor's rectangular perimeter.
      */

      Bounds bounds = this.getBounds();
      double left = bounds.getLeft();
      double right = bounds.getRight();
      double top = bounds.getTop();
      double bottom = bounds.getBottom();

      Tuple[] points = { new Tuple(left, bottom), new Tuple(left, top),
         new Tuple(right, top), new Tuple(right, bottom) };

      return new Polygon(points);
      }
   }
