
import Tuple;
import Dimensions;
import Rectangle;

public class Bounds
   {
   /*
   Define an object to represent a rectangular area.
   */

   //this bounds object covers a rectangular area
   private Rectangle rectangle;

   public Bounds(Rectangle rectangle)
      {
      /*
      Initialize this bounds with the given values.
      */

      this.rectangle = new Rectangle(
         new Tuple(rectangle.getX(), rectangle.getY()),
         new Dimensions(rectangle.getWidth(), rectangle.getHeight()));
      }

   public final double getWidth()
      {
      /*
      Report this bounds' width.
      */

      return rectangle.getWidth();
      }

   public final double getHeight()
      {
      /*
      Report this bounds' height.
      */

      return rectangle.getHeight();
      }

   public final double getLeft()
      {
      /*
      Report this bounds' leftmost bound.
      */

      return rectangle.getX();
      }

   public final double getRight()
      {
      /*
      Report this bounds' rightmost bound.
      */

      return this.getLeft() + this.getWidth();
      }

   public final double getTop()
      {
      /*
      Report this bounds' topmost bound.
      */

      return rectangle.getY();
      }

   public final double getBottom()
      {
      /*
      Report this bounds' bottommost bound.
      */

      return this.getTop() + this.getHeight();
      }

   public final boolean intersects(Bounds bounds)
      {
      /*
      Check whether this bounds intersects the given bounds.
      */

      return (this.xIntersects(bounds) && this.yIntersects(bounds));
      }

   private final boolean xIntersects(Bounds bounds)
      {
      /*
      Check whether this bounds' x-range
      intersects the given bounds' x-range.
      */

      return ((bounds.getLeft() <= this.getRight()) &&
         (this.getLeft() <= bounds.getRight()));
      }

   private final boolean yIntersects(Bounds bounds)
      {
      /*
      Check whether this bounds' y-range
      intersects the given bounds' y-range.
      */

      return ((bounds.getTop() <= this.getBottom()) &&
         (this.getTop() <= bounds.getBottom()));
      }

   public String toString()
      {
      /*
      Report this bounds string representation.
      */

      return "[" + getLeft() + "," + getRight() + "," +
         getTop() + "," + getBottom() + "]";
      }
   }
