
import java.lang.Runnable;
import java.lang.Thread;

public class Clock implements Runnable
   {
   /*
   Define an object to regularly tick() its creator.
   */

   //this clock was created by some clock observable
   private final ClockObservable creator;

   //this clock runs in its own thread
   private Thread thread;
   private boolean running;

   //this clock ticks some number of times per second;
   //to do so it repeatedly pauses for a few milliseconds every second
   private int ticksPerSecond;
   private int pauseTimeInMilliseconds;

   //all clocks have a default number of ticks per second
   private final static int DEFAULT_TICKS_PER_SECOND = 25;
   private final static int MILLISECONDS_PER_SECOND = 1000;

   public Clock(ClockObservable creator)
      {
      /*
      Initialize this clock with the given value.
      */

      this(creator, Clock.DEFAULT_TICKS_PER_SECOND);
      }

   public Clock(ClockObservable creator, int ticksPerSecond)
      {
      /*
      Initialize this clock with the given values.
      */

      this.creator = creator;
      this.ticksPerSecond = ticksPerSecond;
      pauseTimeInMilliseconds =
         (Clock.MILLISECONDS_PER_SECOND / ticksPerSecond);
      thread = new Thread(this);
      }

   public void start()
      {
      /*
      Start this clock ticking.
      */

      running = true;
      thread.start();
      }

   public void stop()
      {
      /*
      Stop this clock.
      */

      running = false;
      }

   public void run()
      {
      /*
      Regularly ask this clock's creator to execute its tick() method.
      */

      while (running)
         {
         creator.tick();
         pause(pauseTimeInMilliseconds);
         }
      }

   private final void pause(int pauseTimeInMilliseconds)
      {
      /*
      Pause this clock for the given time.
      */

      try { Thread.sleep(pauseTimeInMilliseconds); }
      catch (InterruptedException exception) {}
      }
   }
