
import Polygon;
import Bounds;
import Actor;
import Troupe;
import Theater;

public class CollisionDetector
   {
   /*
   Define an object to detect collisions among a troupe of actors.

   This class cannot have a zero-argument constructor;
   it must exist for a particular theater.
   */

   //this collision detector belongs to a theater
   private final Theater theater;

   //that theater's stage has a polygonal perimeter
   private Polygon perimeter;

   public CollisionDetector(Theater theater)
      {
      /*
      Initialize this collision detector to the given theater.
      */

      this.theater = theater;
      this.perimeter = theater.getPerimeter();
      }

   public void detectCollisions()
      {
      /*
      Alert actors who have collided with a boundary or another actor.
      */

      //ask the theater for the current troupe of actors
      Troupe troupe = theater.getTroupe();

      for (int index1 = 0; index1 < troupe.getSize(); index1++)
         {
         //check for actor-boundary collisions
         Actor actor1 = troupe.getActorAt(index1);
         Polygon polygon1 = actor1.getPolygon();
         if (actor1.isMobile() && !perimeter.contains(polygon1))
            actor1.handleCollision(perimeter);

         //check for actor-actor collisions
         Bounds bounds1 = actor1.getBounds();
         for (int index2 = 0; index2 < index1; index2++)
            {
            Actor actor2 = troupe.getActorAt(index2);
            Bounds bounds2 = actor2.getBounds();
            if ((actor1.isMobile() || actor2.isMobile()) &&
               bounds1.intersects(bounds2))
               {
               actor1.handleCollision(actor2);
               actor2.handleCollision(actor1);
               }
            }
         }
      }
   }
