
import Tuple;
import Line;
import LineSegment;

import java.util.Vector;

public class Polygon
   {
   /*
   Define an object to represent a polygon.
   */

   //this polygon is made up of a set of ordered line segments
   //connecting a set of points
   private LineSegment[] edges;
   private Tuple[] points;

   public Polygon(Tuple[] points)
      {
      /*
      Initialize this polygon.
      */

      //save the points
      this.points = new Tuple[points.length];
      for (int index = 0; index < points.length; index++)
         this.points[index] =
            new Tuple(points[index].getX(), points[index].getY());

      //convert the given points to line segments
      edges = new LineSegment[points.length];
      for (int index = 0; index < points.length - 1; index++)
         edges[index] =
            new LineSegment(points[index], points[index + 1]);
      edges[points.length - 1] =
         new LineSegment(points[points.length - 1], points[0]);
      }

   public final int getSize()
      {
      /*
      Report this polygon's number of edges.
      */

      return edges.length;
      }

   public final Tuple[] getPoints()
      {
      /*
      Report an array of this polygon's points.
      */

      Tuple[] points = new Tuple[this.getSize()];
      for (int index = 0; index < this.getSize(); index++)
         {
         double x = this.points[index].getX();
         double y = this.points[index].getY();
         points[index] = new Tuple(x, y);
         }

      return points;
      }

   public final boolean contains(Tuple point)
      {
      /*
      Report whether this polygon contains the given point.
      */

      //make a new horizontal line through the given point
      Line horizon =
         new Line(point, new Tuple(point.getX() + 1, point.getY()));

      //calculate how many times it intersects the polygon's edges
      //to the left of the given point
      int count = 0;
      for (int index = 0; index < this.getSize(); index++)
         {
         if (!edges[index].isHorizontal())
            {
            //find the intersection of the two as lines first
            Tuple intersection = ((Line) edges[index]).intersects(horizon);
            if ((intersection != null) &&
               edges[index].contains(intersection) &&
               (intersection.getX() <= point.getX()))
               count++;
            }
         }

      //the point is outside if the left-intersection count is odd
      return ((count % 2) == 1);
      }

   public final boolean contains(Polygon polygon)
      {
      /*
      Report whether this polygon contains the given one.
      */

      Tuple[] points = polygon.getPoints();

      //see whether every point of the polygon is inside this one
      int index = 0; boolean in = true;
      while ((index < points.length) && in)
         {
         if (!this.contains(points[index]))
            in = false;
         index++;
         }

      return in;
      }

   public final boolean intersects(Polygon polygon)
      {
      /*
      Report whether this polygon intersects the given one.
      */

      return (this.containsAny(polygon) || polygon.containsAny(this));
      }

   public final boolean containsAny(Polygon polygon)
      {
      /*
      Report whether this polygon contains any point of the given polygon.
      */

      Tuple[] points = polygon.getPoints();

      int index = 0; boolean in = false;
      while ((index < points.length) && !in)
         {
         if (this.contains(points[index]))
            in = true;
         index++;
         }

      return in;
      }

   public final LineSegment[] intersects(Line line)
      {
      /*
      Report an array of this polygon's edges the given line intersects.
      */

      Vector hits = new Vector();

      for (int index = 0; index < this.getSize(); index++)
         if (edges[index].intersects(line) != null)
            hits.addElement(edges[index]);

      LineSegment[] hitEdges = new LineSegment[hits.size()];
      for (int index = 0; index < hits.size(); index++)
         hitEdges[index] = (LineSegment) hits.elementAt(index);

      return hitEdges;
      }

   public final LineSegment nearestIntersection(Line line)
      {
      /*
      Report this polygon's edge that first intersects the given line
      (measuring from a special point on the line).
      */

      //find all intersections
      LineSegment[] hitEdges = this.intersects(line);

      if (hitEdges.length == 0)
         return null;

      Tuple endPoint = line.getPoint();
      int indexOfNearest = 0;
      Tuple intersection = hitEdges[0].intersects(line);
      double minimumDistance = endPoint.distanceTo(intersection);
      for (int index = 1; index < hitEdges.length; index++)
         {
         intersection = hitEdges[index].intersects(line);
         double distance = endPoint.distanceTo(intersection);
         if (distance < minimumDistance)
            {
            minimumDistance = distance;
            indexOfNearest = index;
            }
         }

      return hitEdges[indexOfNearest];
      }

   public String toString()
      {
      /*
      Report this polygon's string representation.
      */

      String string = "";
      for (int index = 0; index < this.getSize(); index++)
            string += edges[index] + "  ";

      return string;
      }
   }
